# -*- coding: utf-8 -*-
# HiveMind - Distributed mind map editor for Maemo 5 platform
# Copyright (C) 2010-2011 HiveMind developers
#
# HiveMind is the legal property of its developers, whose names are
# noticed in @author or @authors annotations at the beginning of each
# module or class.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA

'''
Dialogs for Desktop platform
'''

from PyQt4.QtGui import QToolBar, QIcon, QTabWidget, QGridLayout, \
QHBoxLayout, QPalette, QTextBrowser, QFrame, QLabel, QDialogButtonBox, \
QVBoxLayout, QGroupBox, QWidget, QPushButton, QKeySequence
from PyQt4.QtCore import QSize, Qt
from hivemind.dialogs import EditNodeDialog, EditLabelDialog, IconDialog, \
AboutDialog, EditEdgeDialog, SettingsDialog, NetworkConnectionDialog, \
AffiliationDialog, NetworkSettingsDialog, NetworkClientDialog
from hivemind.gui_widgets import FindToolBar
from hivemind.attribute import adaptCallable
import hivemind.__version__ as __version__


class DesktopEditNodeDialog(EditNodeDialog):
    '''
    Edit node dialog for Desktop platform

    @author: Alexander Kulikov
    '''
    def __init__(self, node, parent = None):
        '''
        Initialize dialog with specific node
        @type node: Node
        @type parent: QWidget
        '''
        EditNodeDialog.__init__(self, node, parent)
        self._initComponents()
        self._enableSourceTab(self._isHtml)

    def _initComponents(self):
        '''Form initialization'''
        #pylint: disable = W0201
        self.__tabWidget = QTabWidget(self)
        self.__textIndex = self.__tabWidget.addTab(self._editor, self.tr('Text'))
        self.__htmlIndex = self.__tabWidget.addTab(self._htmlEditor, 'Html')
        mainLayout = QVBoxLayout()
        mainLayout.addWidget(self._createToolbar())
        mainLayout.addWidget(self.__tabWidget)
        lastLineLayout = QHBoxLayout()
        lastLineLayout.addWidget(self._propagateFormatting)
        lastLineLayout.addStretch()
        lastLineLayout.addWidget(self._buttonBox)
        mainLayout.addLayout(lastLineLayout)
        QWidget.setTabOrder(self._editor, self.__tabWidget)
        self._textHtmlAction.toggled.connect(adaptCallable(self._enableSourceTab))
        self.__tabWidget.currentChanged.connect(adaptCallable(self._changedMode))
        self.setLayout(mainLayout)
        self._editor.setFocus()

    def _createToolbar(self):
        '''
        Create and return dialog toolbar
        @rtype QToolBar
        '''
        toolbar = QToolBar(self)
        toolbar.addWidget(self._comboFont)
        toolbar.addWidget(self._comboSize)
        self._boldAction.setToolTip(self.tr('Bold'))
        self._boldAction.setIcon(QIcon.fromTheme('format-text-bold'))
        toolbar.addAction(self._boldAction)
        self._italicAction.setToolTip(self.tr('Italic'))
        self._italicAction.setIcon(QIcon.fromTheme('format-text-italic'))
        toolbar.addAction(self._italicAction)
        self._underlineAction.setToolTip(self.tr('Underline'))
        self._underlineAction.setIcon(QIcon.fromTheme('format-text-underline'))
        toolbar.addAction(self._underlineAction)
        self._textHtmlAction.setToolTip(self.tr('HTML'))
        self._textHtmlAction.setIcon(QIcon.fromTheme('text-html'))
        toolbar.addAction(self._textHtmlAction)
        self._colorAction.setToolTip(self.tr('Color'))
        self._colorAction.setText(self.tr('Color'))
        toolbar.addAction(self._colorAction)
        self._backgroundAction.setToolTip(self.tr('Background'))
        self._backgroundAction.setText(self.tr('Background'))
        toolbar.addAction(self._backgroundAction)
        self._clearFormatAction.setToolTip(self.tr('Clear format'))
        self._clearFormatAction.setIcon(QIcon.fromTheme('edit-clear'))
        toolbar.addAction(self._clearFormatAction)
        return toolbar

    def _changedMode(self, isSourceMode):
        '''
        Modificate dialog when html source was selected
        @type isSourceMode: bool
        '''
        self._boldAction.setDisabled(isSourceMode == self.__htmlIndex)
        self._italicAction.setDisabled(isSourceMode == self.__htmlIndex)
        self._underlineAction.setDisabled(isSourceMode == self.__htmlIndex)
        self._colorAction.setDisabled(isSourceMode == self.__htmlIndex)
        self._backgroundAction.setDisabled(isSourceMode == self.__htmlIndex)
        self._clearFormatAction.setDisabled(isSourceMode == self.__htmlIndex)
        self._isHtmlSourceMode = (isSourceMode == self.__htmlIndex)
        if isSourceMode == self.__htmlIndex:
            self._htmlEditor.setPlainText(self._editor.toHtml())
        else:
            self._editor.setHtml(self._htmlEditor.toPlainText())

    def _enableSourceTab(self, isHtml):
        '''
        Enable html source tab
        @type isHtml: bool
        '''
        self.__tabWidget.setTabEnabled(self.__htmlIndex, isHtml)

class DesktopEditLabelDialog(EditLabelDialog):
    '''
    Edit label dialog for Desktop platform

    @author: Oleg Kandaurov
    '''
    def __init__(self, node, parent = None):
        '''
        Initialize dialog with specific node
        @type node: Node
        @type parent: QWidget
        '''
        EditLabelDialog.__init__(self, node, parent)
        self._initComponents()

    def _initComponents(self):
        '''Form initialization'''
        #create toolbar
        self._boldAction.setToolTip(self.tr('Bold'))
        self._boldAction.setIcon(QIcon.fromTheme('format-text-bold'))
        self._italicAction.setToolTip(self.tr('Italic'))
        self._italicAction.setIcon(QIcon.fromTheme('format-text-italic'))
        self._underlineAction.setToolTip(self.tr('Underline'))
        self._underlineAction.setIcon(QIcon.fromTheme('format-text-underline'))
        self._layoutComponents(self._editor)


class DesktopIconDialog(IconDialog):
    '''
    Icon selection dialog for Desktop platform

    @author: Alexander Kulikov
    '''

    def __init__(self, node, parent = None):
        '''
        Initialize with specific node
        @type node: Node
        @type parent: QWidget
        '''
        IconDialog.__init__(self, node, parent)
        self._initComponents()

    def _initComponents(self):
        '''Form initialization'''
        self.setMinimumWidth(410)
        self._iconTable.setIconSize(QSize(16, 16))
        # setup icon list scene
        self._listScene.setItemSize(32)
        self._listScene.setIconSize(16)
        self._listScene.setHorizontalSpacing(4)
        self._iconList.setMinimumHeight(self._listScene.height() + 8)
        self._iconList.setMaximumHeight(self._listScene.height() + 8)
        groupList = QGroupBox(self.tr('Selected icons'))
        groupTable = QGroupBox(self.tr('Available icons'))
        layout = QGridLayout()
        layout.addWidget(self._iconList, 0, 0)
        groupList.setLayout(layout)
        layout = QGridLayout()
        layout.addWidget(self._iconTable, 0, 0)
        groupTable.setLayout(layout)
        layout = QGridLayout()
        layout.addWidget(groupList, 0, 0, 1, 6)
        layout.addWidget(groupTable, 1, 0, 2, 6)
        layout.addWidget(self._buttonBox, 3, 0, 1, 6)
        self.setLayout(layout)


class DesktopAboutDialog(AboutDialog):
    '''
    About dialog for Desktop platform

    @author: Oleg Kandaurov, Alexander Kulikov
    '''
    def __init__(self, parent = None):
        AboutDialog.__init__(self, parent)

    def _createComponents(self):
        '''Create components'''
        #pylint: disable = W0201
        self._logo.setPixmap(self._logoImage.scaled(QSize(72, 72)))
        palette = self.palette()
        palette.setColor(QPalette.Base, self.palette().color(QPalette.Window))
        self.setPalette(palette)
        self._tabWidget = QTabWidget()
        about = QTextBrowser()
        about.setFrameStyle(QFrame.NoFrame)
        about.setOpenExternalLinks(True)
        about.setHtml('%s - %s<br>%s<br><br>%s<br><br>%s<br>%s<br>' %
                (self._appName, self._appDescription, self._copyright, self._license,
                self._development, self._linkTemplate % (self._fructLink, self._fruct)))
        self._tabWidget.addTab(about, self.tr('&About'))
        authors = QTextBrowser()
        authors.setFrameStyle(QFrame.NoFrame)
        authors.setOpenExternalLinks(True)
        bugReport = unicode(self.tr('Please use %s to report bugs')) % \
                (self._linkTemplate % (self._bugLink, self._siteLink))
        authors.setHtml('%s<br>' % (bugReport) +
                self._generateInfo(self._authorInfo, 'AndreyVasilev') +
                self._generateInfo(self._authorInfo, 'IlyaParamonov') +
                self._generateInfo(self._authorInfo, 'OlegKandaurov') +
                self._generateInfo(self._authorInfo, 'AlexanderKulikov') +
                self._generateInfo(self._authorInfo, 'EldarMamedov') +
                self._generateInfo(self._authorInfo, 'AndreyGolovchenko'))
        self._tabWidget.addTab(authors, self.tr('A&uthors'))
        thanks = QTextBrowser()
        thanks.setFrameStyle(QFrame.NoFrame)
        thanks.setOpenExternalLinks(True)
        thanks.setHtml(self._generateInfo(self._participantInfo, 'NicuBuculei') +
                self._generateInfo(self._participantInfo, 'Mentalrey'))
        self._tabWidget.addTab(thanks, self.tr('&Thanks To'))
        self._name = QLabel('<h2>%s</h2>' % self._appName)
        self._version = QLabel('<h4>%s</h4>' % self.tr('Version %s') % __version__.VERSION)

    def _initComponents(self):
        '''Form initialization'''
        self.setMinimumWidth(500)
        self.setMinimumHeight(300)
        close = QDialogButtonBox(QDialogButtonBox.Close)
        headerInfoLayout = QVBoxLayout()
        headerInfoLayout.addWidget(self._name)
        headerInfoLayout.addWidget(self._version)
        headerInfoLayout.addWidget(self._info)
        headerInfoLayout.setAlignment(Qt.AlignTop)
        headerLayout = QHBoxLayout()
        headerLayout.addWidget(self._logo)
        headerLayout.addLayout(headerInfoLayout)
        headerLayout.setAlignment(Qt.AlignLeft)
        mainLayout = QVBoxLayout()
        mainLayout.addLayout(headerLayout)
        mainLayout.addWidget(self._tabWidget)
        mainLayout.addWidget(close)
        self.setLayout(mainLayout)
        close.rejected.connect(self.close)

    @classmethod
    def _generateInfo(cls, info, name):
        '''
        Generate formatted information about person
        @type info: dict
        @type name: str
        '''
        infoTemplate = ('<br>%s<br>&nbsp;&nbsp;<a href="%s">%s</a><br>&nbsp;&nbsp;%s<br>')
        return infoTemplate % (info[name]['name'], info[name]['site'], info[name]['site'],
                info[name]['role'])


class DesktopEditEdgeDialog(EditEdgeDialog):
    '''
    Dialog for editing decoration and style of a joint link for Desktop platform
    @author: Oleg Kandaurov
    '''
    def __init__(self, node = None, parent = None):
        '''
        Initialize dialog with specific node
        @type node: Node
        '''
        EditEdgeDialog.__init__(self, node, parent)
        self._initComponents()


class DesktopSettingsDialog(SettingsDialog):
    '''
    Settings dialog for Desktop platform
    @author: Oleg Kandaurov
    '''
    def __init__(self, parent = None):
        '''
        @type parent: QWidget
        '''
        SettingsDialog.__init__(self, parent)
        self._createTab(self.tr('General'), ['locale', 'confirmDelete', 'defaultZoomLevel',
                'maxNodeTextWidth'])
        self._createTab(self.tr('Node'), ['defaultFont', 'defaultNodeTextColor',
                'defaultNodeBackgroundColor', 'defaultSelectedNodeBackgroundColor'])
        self._createTab(self.tr('Edge && label'), ['defaultLabelFont', 'defaultLabelColor',
                'defaultEdgeWidth', 'defaultEdgeColor'])
        self._createTab(self.tr('Autosave'), ['autosaveEnabled', 'autosaveInterval'])
        self._createTab(self.tr('Shortcuts'), ['shortcuts'], True)


class DesktopAffiliationDialog(AffiliationDialog):
    '''
    Affiliation dialog for Desktop platform
    @author: Oleg Kandaurov
    '''
    def __init__(self, model, parent = None):
        AffiliationDialog.__init__(self, model, parent)


class DesktopNetworkSettingsDialog(NetworkSettingsDialog):
    '''
    Network settings dialog for Desktop platfrom
    @author: Oleg Kandaurov
    '''

    def __init__(self, parent = None):
        NetworkSettingsDialog.__init__(self, parent)


class DesktopNetworkClientDialog(NetworkClientDialog):
    '''
    Client dialog for Desktop platfrom
    @author: Oleg Kandaurov
    '''

    def __init__(self, rosterModel, parent = None):
        NetworkClientDialog.__init__(self, rosterModel, parent)


class DesktopNetworkConnectionDialog(NetworkConnectionDialog):
    '''
    Connection dialog for Desktop platfrom
    @author: Oleg Kandaurov
    '''

    def __init__(self, parent = None):
        NetworkConnectionDialog.__init__(self, parent)


class DesktopFindToolBar(FindToolBar):
    '''
    Find tool bar for Desktop platform
    '''
    def __init__(self, controller, parent = None):
        '''
        @type controller: MainWindowController
        @type parent: QWidget
        '''
        FindToolBar.__init__(self, controller, parent)
        self.setAllowedAreas(Qt.TopToolBarArea | Qt.BottomToolBarArea)
        self._createComponents()
        self._initComponents()

    def _createComponents(self):
        '''
        Create components
        '''
        #pylint: disable = W0201
        self.__prevButton = QPushButton(self.tr('&Previous'))
        self.__nextButton = QPushButton(self.tr('&Next'))
        self.__closeButton = QPushButton(self.tr('&Close'))
        self.__closeButton.setIcon(QIcon.fromTheme('window-close'))
        self.__prevButton.setIcon(QIcon.fromTheme('go-previous'))
        self.__nextButton.setIcon(QIcon.fromTheme('go-next'))
        self.__prevButton.setShortcut(QKeySequence.FindPrevious)
        self.__nextButton.setShortcut(QKeySequence.FindNext)
        self.__nextButton.clicked.connect(self._finder.nextFoundNode)
        self.__prevButton.clicked.connect(self._finder.prevFoundNode)
        self.__closeButton.clicked.connect(self.close)

    def _initComponents(self):
        '''
        Init components
        '''
        FindToolBar._createComponents(self)
        self._findLineEdit.setFixedWidth(self._findLineEdit.sizeHint().width())
        mainLayout = QHBoxLayout()
        mainLayout.addWidget(self._findLabel)
        mainLayout.addWidget(self._findLineEdit)
        mainLayout.addWidget(self.__prevButton)
        mainLayout.addWidget(self.__nextButton)
        mainLayout.addWidget(self._highlightCheckBox)
        mainLayout.addWidget(self._caseSensitiveCheckBox)
        mainLayout.addStretch()
        mainLayout.addWidget(self._statusLabel)
        mainLayout.addStretch()
        mainLayout.addWidget(self.__closeButton)
        self._findWidget.setLayout(mainLayout)
        self.addWidget(self._findWidget)
